from __future__ import unicode_literals
import os
import json


TEST_DIR = os.path.dirname(__file__)
SAMPLES_DIR = os.path.join(TEST_DIR, 'samples')


def get_sample_path(sample_filename):
    return os.path.join(SAMPLES_DIR, sample_filename)


def get_sample(sample_filename):
    with open(get_sample_path(sample_filename)) as f:
        return json.load(f)


REQUIRE_OBJECT_TYPE = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
}

REQUIRING_SOME_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
    "required": ['something'],
    "properties": {
        "something": {"type": "string"},
    }
}

REQUIRING_NULL_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
    "required": ['something'],
    "properties": {
        "something": {"type": "null"},
    }
}

REQUIRING_NUMBER_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
    "required": ['number_of_followers'],
    "properties": {
        "number_of_followers": {"type": "number"},
    }
}

REQUIRING_BOOLEAN_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
    "required": ['is_this_on'],
    "properties": {
        "is_this_on": {"type": "boolean"},
    }
}

REQUIRING_ANOTHER_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ['another'],
    "properties": {
        "another": {"type": "string"},
    }
}

REQUIRING_SOME_OBJECT_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    'type': 'object',
    "required": ['something'],
    "properties": {
        "something": {"type": "object"},
    }
}


REQUIRING_SOME_PROPERTY_WITH_NESTED_REQUIRED_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["something"],
    "properties": {
        "something": {
            "type": "object",
            "required": ['nested_required'],
            "properties": {
                "nested_required": {"type": "string"},
            }
        },
    }
}

REQUIRING_SOME_PROPERTY_WITH_NESTED_OPTIONAL_AND_REQUIRED_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["something"],
    "properties": {
        "something": {
            "type": "object",
            "required": ['nested_required'],
            "properties": {
                "nested_required": {"type": "string"},
                "nested_optional": {"type": "string"},
            }
        },
    }
}

REQUIRING_AN_ARRAY_PROPERTY = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["a_list"],
    "properties": {
        "a_list": {
            "type": "array",
            "items": {"type": "string"},
        }
    }
}

REQUIRING_AN_ARRAY_PROPERTY_PLUS_ANOTHER = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["a_list", "another_list"],
    "properties": {
        "a_list": {
            "type": "array",
            "items": {"type": "string"},
        },
        "another_list": {
            "type": "array",
            "items": {"type": "number"},
        },
    }
}


REQUIRING_AN_ARRAY_PROPERTY_WITH_TUPLE_VALIDATION = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["array_with_tuple"],
    "properties": {
        "array_with_tuple": {
            "type": "array",
            "items": [
                {"type": "string"},
                {"type": "string"},
            ],
        },
    }
}

REQUIRING_AN_ARRAY_PROPERTY_CONFLICTING_WITH_TUPLE = {
    '$schema': u'http://json-schema.org/draft-04/schema',
    "type": "object",
    "required": ["array_with_tuple"],
    "properties": {
        "array_with_tuple": {
            "type": "array",
            "items": {"type": "string"},
        },
    }
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function
from skinfer.draft4_generator import IncompleteDraft4SchemaGenerator
import unittest
from tests import fixtures


def generate_schema(sample):
    return IncompleteDraft4SchemaGenerator(sample).to_dict()


class TestDraft4SchemaGenerator(unittest.TestCase):
    def test_empty_object(self):
        self.assertEqual(generate_schema({}), fixtures.REQUIRE_OBJECT_TYPE)

    def test_simple_object(self):
        self.assertEqual(generate_schema({"something": "ai"}),
                         fixtures.REQUIRING_SOME_PROPERTY)

    def test_object_with_nested_properties(self):
        self.assertEqual(generate_schema({"something": {"nested_required": "1"}}),
                         fixtures.REQUIRING_SOME_PROPERTY_WITH_NESTED_REQUIRED_PROPERTY)

    def assertIn(self, a, b):
        """polyfill for python 2.6"""
        self.assertTrue(a in b, "%r not found in %r" % (a, b))

    def assertSchemaEqual(self, schema1, schema2):
        self.assertEqual(len(schema1), len(schema2))
        self.assertEqual(schema1.get('type'), schema2.get('type'))

        schema_type = schema1.get('type')

        if schema_type:
            for k, v in schema1.get('properties').items():
                self.assertIn(k, schema2['properties'])
            self.assertEqual(sorted(schema1.get('required')), sorted(schema2.get('required')))

    def test_with_linkedin_minimal_example(self):
        data = fixtures.get_sample('minimal-1.json')

        expected = {
            '$schema': u'http://json-schema.org/draft-04/schema',
            'properties': {
                u'also_viewed': {'items': {'type': 'string'}, 'type': 'array'},
                u'family_name': {'type': 'string'},
                u'full_name': {'type': 'string'},
                u'given_name': {'type': 'string'},
                u'headline': {'type': 'string'},
                u'linkedin_id': {'type': 'string'},
                u'locality': {'type': 'string'},
                u'num_connections': {'type': 'string'},
                u'updated': {'type': 'string'},
                u'url': {'type': 'string'}},
            'required': [
                u'family_name',
                u'updated',
                u'linkedin_id',
                u'locality',
                u'headline',
                u'num_connections',
                u'url',
                u'also_viewed',
                u'given_name',
                u'full_name'],
            'type': 'object'
        }
        self.assertSchemaEqual(generate_schema(data), expected)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import itertools
import sys


def merge_property_list(first_properties, second_properties):
    result = {}

    for key, value in first_properties.items():
        if key in second_properties:
            result[key] = _merge_schema(value, second_properties[key])
        else:
            result[key] = value

    for key, value in second_properties.items():
        if key not in result:
            result[key] = value

    return result


def get_reserved_keys(schema_type):
    if (schema_type not in SCHEMA_TYPES or
            'reserved_keys' not in SCHEMA_TYPES[schema_type]):
        mesg = "Missing implementation for schema type: %s" % schema_type
        raise NotImplementedError(mesg)

    return SCHEMA_TYPES[schema_type]['reserved_keys']


def copy_nonreserved_keys(first, second):
    reserved_keys = get_reserved_keys(first.get('type'))

    return ((key, value)
            for key, value in itertools.chain(first.items(), second.items())
            if key not in reserved_keys)


def merge_objects(first, second):
    required = list(set(first.get('required', [])) &
                    set(second.get('required', [])))

    result = {
        'type': 'object',
        'properties': merge_property_list(first.get('properties', {}),
                                          second.get('properties', {})),
    }

    if required:
        result['required'] = required

    result.update(copy_nonreserved_keys(first, second))

    return result


def min_or_none(val1, val2):
    """Returns min(val1, val2) returning None only if both values are None"""
    return min(val1, val2, key=lambda x: sys.maxint if x is None else x)


def max_or_none(val1, val2):
    """Returns max(val1, val2) returning None only if both values are None"""
    return max(val1, val2, key=lambda x: -sys.maxint if x is None else x)


def merge_strings(first, second):
    result = {'type': 'string'}

    result.update(copy_nonreserved_keys(first, second))

    minLength = min_or_none(first.get('minLength'), second.get('minLength'))
    if minLength:
        result['minLength'] = minLength

    maxLength = max_or_none(first.get('maxLength'), second.get('maxLength'))
    if maxLength:
        result['maxLength'] = maxLength

    return result


def merge_numbers(first, second):
    return {"type": "number"}


def merge_booleans(first, second):
    return {"type": "boolean"}


def merge_nulls(first, second):
    return {"type": "null"}


def merge_arrays(first, second):
    def is_schema_tuple(item):
        return isinstance(item, list)

    def are_json_schema_tuples(first_items, second_items):
        return all([
            is_schema_tuple(first_items),
            is_schema_tuple(second_items),
            len(first_items) == len(second_items)
        ])

    def merge_tuples(first_items, second_items):
        return [_merge_schema(e1, e2) for e1, e2 in zip(first_items, second_items)]

    def merge_array_list_with_array_tuple(first_items, second_items):
        tuple_items, merged = first_items, second_items

        if is_schema_tuple(merged):
            tuple_items, merged = merged, tuple_items

        for schema in tuple_items:
            merged = _merge_schema(merged, schema)

        return merged

    def merge_items(first_items, second_items):
        if not (first_items and second_items):
            return None

        if are_json_schema_tuples(first_items, second_items):
            return merge_tuples(first_items, second_items)

        if is_schema_tuple(first_items) or is_schema_tuple(second_items):
            return merge_array_list_with_array_tuple(first_items, second_items)

        return _merge_schema(first_items, second_items)

    items = merge_items(first.get('items'), second.get('items'))

    result = {
        'type': 'array',
    }

    if items:
        result['items'] = items

    result.update(copy_nonreserved_keys(first, second))

    return result


def merge_with_any_of(first, second):
    first_any_of = first['anyOf'] if 'anyOf' in first else [first]
    second_any_of = second['anyOf'] if 'anyOf' in second else [second]

    any_of = []
    for schema in first_any_of + second_any_of:
        if schema not in any_of:
            any_of.append(schema)

    return {"anyOf": any_of}


def _merge_schema(first, second):
    if first.get('type') != second.get('type'):
        return merge_with_any_of(first, second)

    if 'anyOf' in first or 'anyOf' in second:
        return merge_with_any_of(first, second)

    schema_type = first.get('type')

    if schema_type not in SCHEMA_TYPES:
        raise NotImplementedError("Type %s is not yet supported" % schema_type)

    merge_function = SCHEMA_TYPES[schema_type]['merge_function']
    return merge_function(first, second)


def merge_schema(first, second):
    """Returns the result of merging the two given schemas.
    """
    if not (type(first) == type(second) == dict):
        raise ValueError("Argument is not a schema")

    if not (first.get('type') == second.get('type') == 'object'):
        raise NotImplementedError("Unsupported root type")

    return merge_objects(first, second)


SCHEMA_TYPES = {
    'object': {
        'merge_function': merge_objects,
        'reserved_keys': set(['type', 'properties', 'required']),
    },
    'string': {
        'merge_function': merge_strings,
        'reserved_keys': set(['type', 'minLength', 'maxLength']),
    },
    'array': {
        'merge_function': merge_arrays,
        'reserved_keys': set(['type', 'items']),
    },
    'number': {
        'merge_function': merge_numbers,
    },
    'boolean': {
        'merge_function': merge_booleans,
    },
    'null': {
        'merge_function': merge_nulls,
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import unittest
import json
from tests import fixtures
from skinfer import schema_inferer
import subprocess


class TestJsonSchemaInferer(unittest.TestCase):

    def test_run_infering_schema_for_yelp_samples(self):
        sample1 = fixtures.get_sample('sample1-yelp.json')
        sample2 = fixtures.get_sample('sample2-yelp.json')
        schema = schema_inferer.generate_and_merge_schemas([sample1, sample2])
        self.assertTrue(type(schema) == dict)

    def test_load_json_samples(self):
        # given:
        sample1 = fixtures.get_sample_path('sample1-yelp.json')
        sample2 = fixtures.get_sample_path('sample2-yelp.json')

        # when:
        samples = list(schema_inferer.load_samples_from_json([sample1, sample2]))

        # then:
        self.assertEquals(2, len(samples))

    def test_load_jsonlines_samples(self):
        # given:
        infile = fixtures.get_sample_path('jsonlines.jsonl.gz')

        # when:
        samples = list(schema_inferer.load_samples_from_jsonlines([infile]))

        # then:
        self.assertEquals(3, len(samples))


def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    Backported from Python 2.7 as it's implemented as pure python on stdlib.

    >>> check_output(['/usr/bin/python', '--version'])
    Python 2.6.2
    """
    # FROM: https://gist.github.com/edufelipe/1027906
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output


class TestCasePython26Shim(unittest.TestCase):
    def assertIsNotNone(self, value):
        self.assertFalse(value is None, "%r is not None" % value)

    def assertIn(self, value, seq):
        self.assertTrue(value in seq, "%r is not in %r" % (value, seq))


class TestSchemaInfererScriptTest(TestCasePython26Shim):
    def test_run_with_json_samples_in_separate_files(self):
        # given:
        sample1 = fixtures.get_sample_path('minimal-1.json')
        sample2 = fixtures.get_sample_path('sample2-yelp.json')
        # when:
        output = check_output(['bin/schema_inferer', sample1, sample2])
        # then:
        data = json.loads(output)
        self.assertIsNotNone(data)
        self.assertIn('required', data)
        self.assertIn('properties', data)

    def test_run_with_jsonlines_samples(self):
        # given:
        infile = fixtures.get_sample_path('jsonlines.jsonl')
        # when:
        output = check_output(['bin/schema_inferer', '--jsonlines', infile])
        # then:
        data = json.loads(output)
        self.assertIsNotNone(data)
        self.assertIn('required', data)
        self.assertIn('properties', data)

    def test_run_with_jsonlines_samples_omitting_option(self):
        # given:
        infile = fixtures.get_sample_path('jsonlines.jsonl')
        # when:
        output = check_output(['bin/schema_inferer', infile])
        # then:
        data = json.loads(output)
        self.assertIsNotNone(data)
        self.assertIn('required', data)
        self.assertIn('properties', data)



# -*- coding: utf-8 -*-
"""
Skinfer - tools for inferring and merging JSON schemas
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Basic usage of inferring schemas:

    >>> import skinfer, pprint
    >>> sample1 = {'name': 'Claudio'}
    >>> sample2 = {'name': 'Roberto', 'surname': 'Salazar'}
    >>> merged_schema = skinfer.infer_schema([sample1, sample2])
    >>> pprint.pprint(merged_schema)
    {'$schema': u'http://json-schema.org/draft-04/schema',
      u'properties': {'name': {u'type': u'string'}, 'surname': {'type': 'string'}},
      u'required': ['name'],
      u'type': u'object'}

and merging schemas manually:

    >>> schema1 = skinfer.generate_schema(sample1)
    >>> schema2 = skinfer.generate_schema(sample2)
    >>> final_schema = skinfer.merge_schema(schema1, schema2)
    >>> assert merged_schema == final_schema

"""

from __future__ import absolute_import, division, print_function


__author__ = 'Scrapinghub'
__email__ = 'info@scrapinghub.com'
__version__ = '0.1.2'


from .json_schema_merger import merge_schema  # NOQA
from .schema_inferer import generate_schema_for_sample as generate_schema  # NOQA
from .schema_inferer import generate_and_merge_schemas as infer_schema  # NOQA



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from skinfer.json_schema_merger import merge_schema
import unittest
from tests import fixtures


def recursive_sort(obj):
    """
    Recursively sort list or dict nested lists
    """

    if isinstance(obj, dict):
        for key, val in obj.iteritems():
            obj[key] = recursive_sort(val)
        _sorted = obj

    elif isinstance(obj, list):
        new_list = []
        for val in obj:
            new_list.append(recursive_sort(val))
        _sorted = sorted(new_list)

    else:
        _sorted = obj

    return _sorted


class TestJsonSchemaMerger(unittest.TestCase):
    def check_merge_result(self, first, second, expected):
        first_copy = first.copy()
        second_copy = second.copy()

        merged = merge_schema(first, second)
        merged_reversed = merge_schema(second, first)

        self.assertEqual(merged, expected)
        self.assertEqual(merged_reversed, expected)
        self.assertEqual(merged, merged_reversed)

        # ensuring we're not touching the originals:
        self.assertEqual(first, first_copy)
        self.assertEqual(second, second_copy)

    def test_merge_required_property_with_empty_object_schema(self):
        self.check_merge_result(
            fixtures.REQUIRE_OBJECT_TYPE,
            fixtures.REQUIRING_SOME_PROPERTY,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string"},
                }
            }
        )

    def test_merge_schemas_with_different_required_properties(self):
        self.check_merge_result(
            fixtures.REQUIRING_SOME_PROPERTY,
            fixtures.REQUIRING_ANOTHER_PROPERTY,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string"},
                    "another": {"type": "string"},
                }
            }
        )

    def test_merge_same_schema_gives_itself(self):
        self.check_merge_result(
            fixtures.REQUIRING_SOME_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY,
        )
        self.check_merge_result(
            fixtures.REQUIRING_SOME_PROPERTY_WITH_NESTED_OPTIONAL_AND_REQUIRED_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY_WITH_NESTED_OPTIONAL_AND_REQUIRED_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY_WITH_NESTED_OPTIONAL_AND_REQUIRED_PROPERTY,
        )
        self.check_merge_result(
            fixtures.REQUIRING_AN_ARRAY_PROPERTY,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY,
        )

    def test_merge_schemas_with_nested_properties(self):
        self.check_merge_result(
            fixtures.REQUIRING_SOME_OBJECT_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY_WITH_NESTED_OPTIONAL_AND_REQUIRED_PROPERTY,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "required": ["something"],
                "properties": {
                    "something": {
                        "type": "object",
                        "properties": {
                            "nested_optional": {"type": "string"},
                            "nested_required": {"type": "string"},
                        }
                    },
                }
            }
        )

    def test_merge_one_schema_with_nested_array_property(self):
        self.check_merge_result(
            fixtures.REQUIRING_SOME_PROPERTY,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "a_list": {
                        "type": "array",
                        "items": {"type": "string"}
                    },
                    "something": {"type": "string"}
                }
            }
        )

    def test_merge_both_schemas_with_nested_array_property(self):
        self.check_merge_result(
            fixtures.REQUIRING_AN_ARRAY_PROPERTY,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY_PLUS_ANOTHER,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "required": ["a_list"],
                "properties": {
                    "a_list": {
                        "type": "array",
                        "items": {"type": "string"},
                    },
                    "another_list": {
                        "type": "array",
                        "items": {"type": "number"},
                    }
                }
            }
        )

    def test_merge_array_with_tuple_validation(self):
        self.check_merge_result(
            fixtures.REQUIRING_AN_ARRAY_PROPERTY_WITH_TUPLE_VALIDATION,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY_WITH_TUPLE_VALIDATION,
            {
                '$schema': 'http://json-schema.org/draft-04/schema',
                "required": ["array_with_tuple"],
                'type': 'object',
                'properties': {
                    'array_with_tuple': {
                        'items': [{'type': 'string'}, {'type': 'string'}],
                        'type': u'array'
                    }
                }
            }
        )

    def test_merge_simple_numbers(self):
        self.check_merge_result(
            fixtures.REQUIRING_NUMBER_PROPERTY,
            fixtures.REQUIRING_NUMBER_PROPERTY,
            {
                '$schema': 'http://json-schema.org/draft-04/schema',
                'type': 'object',
                'required': ['number_of_followers'],
                'properties': {
                    'number_of_followers': {
                        'type': 'number',
                    }
                }
            }
        )

    def test_merge_simple_boolean(self):
        self.check_merge_result(
            fixtures.REQUIRING_BOOLEAN_PROPERTY,
            fixtures.REQUIRING_BOOLEAN_PROPERTY,
            {
                '$schema': 'http://json-schema.org/draft-04/schema',
                'type': 'object',
                'required': ['is_this_on'],
                'properties': {
                    'is_this_on': {
                        'type': 'boolean',
                    }
                }
            }
        )

    def assertSchemaEqual(self, first, second):
        self.assertEqual(recursive_sort(first), recursive_sort(second))

    def test_merge_string_and_null(self):
        merged = merge_schema(
            fixtures.REQUIRING_NULL_PROPERTY,
            fixtures.REQUIRING_SOME_PROPERTY,
        )
        expected = {
            '$schema': 'http://json-schema.org/draft-04/schema',
            'type': 'object',
            'required': ['something'],
            'properties': {
                'something': {
                    'anyOf': [
                        {'type': 'string'},
                        {'type': 'null'},
                    ]
                }
            }
        }
        self.assertSchemaEqual(merged, expected)

    def test_merge_anyof(self):
        schema_with_anyof = {
            '$schema': 'http://json-schema.org/draft-04/schema',
            'type': 'object',
            'required': ['something'],
            'properties': {
                'something': {
                    'anyOf': [
                        {'type': 'string'},
                        {'type': 'null'},
                    ]
                }
            }
        }
        merged = merge_schema(fixtures.REQUIRING_SOME_PROPERTY, schema_with_anyof)
        self.assertSchemaEqual(merged, schema_with_anyof)

    def test_merge_null(self):
        self.check_merge_result(
            fixtures.REQUIRING_NULL_PROPERTY,
            fixtures.REQUIRING_NULL_PROPERTY,
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "required": ["something"],
                "properties": {
                    "something": {"type": "null"},
                }
            }
        )

    def test_merge_tuple_with_loose_array(self):
        self.check_merge_result(
            fixtures.REQUIRING_AN_ARRAY_PROPERTY_WITH_TUPLE_VALIDATION,
            fixtures.REQUIRING_AN_ARRAY_PROPERTY_CONFLICTING_WITH_TUPLE,
            {
                '$schema': 'http://json-schema.org/draft-04/schema',
                "required": ["array_with_tuple"],
                'type': 'object',
                'properties': {
                    'array_with_tuple': {
                        'items': {'type': 'string'},
                        'type': u'array'
                    }
                }
            }
        )

    def test_merge_string_keeping_custom_props(self):
        self.check_merge_result(
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "default": "botemo"},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "description": "Something"},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "description": "Something", "default": "botemo"},
                }
            }
        )

    def test_merge_string_minlength_nolength(self):
        self.check_merge_result(
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string"},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2},
                }
            },
        )

    def test_merge_string_maxlength_nolength(self):
        self.check_merge_result(
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "maxLength": 22},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2, "maxLength": 22},
                }
            },
        )

    def test_merge_string_minlength_maxlength(self):
        self.check_merge_result(
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2, "maxLength": 10},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 4, "maxLength": 12},
                }
            },
            {
                '$schema': u'http://json-schema.org/draft-04/schema',
                "type": "object",
                "properties": {
                    "something": {"type": "string", "minLength": 2, "maxLength": 12},
                }
            }
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function
import json_schema_generator
from json_schema_generator.schema_types import ArrayType, ObjectType


class Type(json_schema_generator.schema_types.Type):
    schema_version = u"http://json-schema.org/draft-04/schema"


class IncompleteDraft4SchemaGenerator(json_schema_generator.SchemaGenerator):
    """Patching json_schema_generator.SchemaGenerator to generate
    Draft4 valid schemas

    See original at:
    https://github.com/perenecabuto/json_schema_generator/blob/master/json_schema_generator/generator.py

    This is a hack, just a copy of the method from the base class with
    a few changes to generate a valid Draft4 schema.

    We should implement a full Draft4SchemaGenerator -- probably upstream.
    """
    def to_dict(self, base_object=None, object_id=None, first_level=True):
        schema_dict = {}

        if first_level:
            base_object = self.base_object
            schema_dict["$schema"] = Type.schema_version

        base_object_type = type(base_object)
        schema_type = Type.get_schema_type_for(base_object_type)

        schema_dict["type"] = schema_type.json_type

        if schema_type == ObjectType and len(base_object) > 0:
            schema_dict["properties"] = {}
            schema_dict["required"] = list(base_object.keys())

            for prop, value in base_object.items():
                schema_dict["properties"][prop] = self.to_dict(value, prop, False)

        elif schema_type == ArrayType and len(base_object) > 0:
            first_item_type = type(base_object[0])
            same_type = all((type(item) == first_item_type for item in base_object))

            if same_type:
                schema_dict['items'] = self.to_dict(base_object[0], 0, False)

            else:
                schema_dict['items'] = []

                for idx, item in enumerate(base_object):
                    schema_dict['items'].append(self.to_dict(item, idx, False))

        return schema_dict



from __future__ import absolute_import, division, print_function, unicode_literals

import json
import gzip
from skinfer.draft4_generator import IncompleteDraft4SchemaGenerator
from skinfer.json_schema_merger import merge_schema


class GzipFileShim(gzip.GzipFile):
    """Python 2.6 Shim
    See https://mail.python.org/pipermail/tutor/2009-November/072959.html
    """
    def __enter__(self):
        if self.fileobj is None:
            raise ValueError("I/O operation on closed GzipFile object")
        return self

    def __exit__(self, *args):
        self.close()


def gzopen(filename):
    if '.gz' in filename:
        return GzipFileShim(filename)

    return open(filename)


def load_samples_from_jsonlines(file_list):
    for filename in file_list:
        with gzopen(filename) as f:
            for line in f:
                yield json.loads(line)


def load_samples_from_json(file_list):
    for filename in file_list:
        with gzopen(filename) as f:
            yield json.load(f)


def generate_schema_for_sample(sample):
    """Returns a schema generated for the given sample.
    """
    return IncompleteDraft4SchemaGenerator(sample).to_dict()


def generate_and_merge_schemas(samples):
    """Iterates through the given samples, generating schemas
    and merging them, returning the resulting merged schema.

    """
    merged = generate_schema_for_sample(next(iter(samples)))

    for sample in samples:
        merged = merge_schema(merged, generate_schema_for_sample(sample))

    return merged



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup


readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = [
    'json-schema-generator >= 0.3'
]

test_requirements = [
    # put package test requirements here
]

setup(
    name='skinfer',
    version='0.1.2',
    description='Simple tool to merge JSON schemas',
    long_description=readme + '\n\n' + history,
    author='Scrapinghub',
    author_email='info@scrapinghub.com',
    url='https://github.com/scrapinghub/skinfer',
    packages=[
        'skinfer',
    ],
    package_dir={'skinfer':
                 'skinfer'},
    scripts=[
        'bin/schema_inferer',
        'bin/json_schema_merger',
    ],
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='skinfer json-schema json schema inferer merger',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
